# -*- coding: utf-8 -*-
# vim: ts=4 sw=4 sts=4 expandtab ai

# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
#from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.decorators import login_required

from models import Medlem, Tilskiping

@login_required
def listing(request):
    tilskipingar = Tilskiping.objects.all()

    return render_to_response('tilskiping/tilskiping_listing.html', {
        'objects': tilskipingar,
    })

@login_required
def detail(request, slug):
    tilskiping = Tilskiping.objects.get(slug=slug)

    if not request.user.is_staff:
        try:
            medlem_prof = request.user.get_profile()
        except ObjectDoesNotExist:
            return render_to_response("error.html", { 'message': """Brukaren din er
            ikkje kopla opp mot ein medlemsprofil. Det må til for å få løyve til å
            sjå ting.""" })

        if not medlem_prof.tilskiping.filter(slug=slug).exists():
            return render_to_response("error.html", { 'message': "Du var ikkje med på denne tilskipinga." })

    medlem = Medlem.objects.filter(tilskiping=tilskiping)
    betalt_count = Medlem.objects.betalande().filter(tilskiping=tilskiping).count()
    teljande_count = Medlem.objects.teljande().filter(tilskiping=tilskiping).count()

    return render_to_response('tilskiping/tilskiping_home.html', {
        'tilskiping': tilskiping,
        'medlem': medlem,
        'betalt_count': betalt_count,
        'teljande_count': teljande_count,
    })



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
import datetime

from django.contrib.auth.models import User
from django.test import Client
from django.test import TestCase
from django.utils import timezone

from medlemssys.medlem import models
from medlemssys.medlem.models import Giro
from medlemssys.medlem.models import Medlem


year = datetime.date.today().year


def lagMedlem(alder, utmeldt=False, har_betalt=False, name="", **kwargs):
    if not name:
        name = str(alder)
        if (har_betalt):
             name += "-betalt"
        if (utmeldt):
             name += "-utmeld"

    medlem = Medlem(fornamn=name, etternamn="E", fodt=year - alder,
                    postnr="5000", **kwargs)

    if (utmeldt):
        medlem.utmeldt_dato = datetime.date.today()

    medlem.save()

    if (har_betalt):
        g = Giro(medlem=medlem,
                 belop=80,
                 innbetalt_belop=80,
                 innbetalt=timezone.now())
        g.save()

    return medlem

def lagTestMedlemar(**extra_kwargs):
    medlemar = {}

    def opprett_og_lagra_medlem(*args, **kwargs):
        kwargs.update(extra_kwargs)
        medlem = lagMedlem(*args, **kwargs)
        medlemar[medlem.fornamn] = medlem
        return medlem

    opprett_og_lagra_medlem(25, utmeldt=False, har_betalt=False)
    opprett_og_lagra_medlem(25, utmeldt=True,  har_betalt=False)
    opprett_og_lagra_medlem(25, utmeldt=False, har_betalt=True)
    opprett_og_lagra_medlem(25, utmeldt=True,  har_betalt=True)

    opprett_og_lagra_medlem(12, utmeldt=False, har_betalt=False)
    opprett_og_lagra_medlem(12, utmeldt=True,  har_betalt=False)
    opprett_og_lagra_medlem(12, utmeldt=False, har_betalt=True)
    opprett_og_lagra_medlem(12, utmeldt=True,  har_betalt=True)

    opprett_og_lagra_medlem(26, utmeldt=False, har_betalt=False)
    opprett_og_lagra_medlem(26, utmeldt=True,  har_betalt=False)
    opprett_og_lagra_medlem(26, utmeldt=False, har_betalt=True)
    opprett_og_lagra_medlem(26, utmeldt=True,  har_betalt=True)

    m = opprett_og_lagra_medlem(23, name="23-betaltifjor")
    m.innmeldt_dato = datetime.date(year - 2, 1, 1)
    Giro(medlem=m,
         belop=80,
         innbetalt_belop=80,
         gjeldande_aar=year - 1,
         oppretta=datetime.date(year - 1, 1, 1),
         innbetalt=datetime.date(year - 1, 1, 1)).save()
    m.save()

    m = opprett_og_lagra_medlem(23, name="23-betaltiforfjor")
    m.innmeldt_dato = datetime.date(year - 2, 1, 1)
    Giro(medlem=m,
         belop=80,
         innbetalt_belop=80,
         gjeldande_aar=year - 2,
         oppretta=datetime.date(year - 2, 1, 1),
         innbetalt=datetime.date(year - 2, 1, 1)).save()
    m.save()

    m = opprett_og_lagra_medlem(23, name="23-innmeldtifjor")
    m.innmeldt_dato = datetime.date(year - 1, 1, 1)
    Giro(medlem=m,
         belop=80,
         innbetalt_belop=0,
         gjeldande_aar=year - 1,
         oppretta=datetime.date(year - 1, 1, 1)).save()
    m.save()

    m = opprett_og_lagra_medlem(23, name="23-innmeldtifjor-utmeldtiaar")
    m.innmeldt_dato = datetime.date(year - 1, 1, 1)
    m.utmeldt_dato = datetime.date(year, 1, 1)
    Giro(medlem=m,
         belop=80,
         innbetalt_belop=0,
         gjeldande_aar=year - 1,
         oppretta=datetime.date(year - 1, 1, 1)).save()
    m.save()

    m = opprett_og_lagra_medlem(23, name="23-innmeldtiforfjor-utmeldtnesteaar")
    m.innmeldt_dato = datetime.date(year - 2, 1, 1)
    m.utmeldt_dato = datetime.date(year + 1, 1, 1)
    m.save()

    m = opprett_og_lagra_medlem(23, name="23-betaltifjor-utmeldtnesteaar")
    m.innmeldt_dato = datetime.date(year - 2, 1, 1)
    m.utmeldt_dato = datetime.date(year + 1, 1, 1)
    Giro(medlem=m,
         belop=80,
         innbetalt_belop=80,
         gjeldande_aar=year - 1,
         oppretta=datetime.date(year - 1, 1, 1),
         innbetalt=datetime.date(year - 1, 1, 1)).save()
    m.save()

    m = opprett_og_lagra_medlem(23, har_betalt=True, name="23-betalt-utmeldtnesteaar")
    m.utmeldt_dato = datetime.date(year + 1, 1, 2)
    m.save()

    # innmeldt og utmeldt i fjor
    # innmeldt i fjor, men utmeldt no
    # utmeldt til neste år

    return medlemar


class MedlemTest(TestCase):

    def test_utmeldt_delete_unpaid_giro_on_denormalized(self):
        # Unpaid giros are deleted after member is utmeldt and
        # update_denormalized_fields has run
        m = lagMedlem(29, har_betalt=True)
        g = Giro(medlem=m, belop=10)
        g.save()
        self.assertEqual(
            [g.belop for g in m.giroar.all()], [10, 80])
        m.utmeldt_dato = datetime.date.today()
        m.save()
        models.update_denormalized_fields()
        self.assertEqual(
            [g.belop for g in m.giroar.all()], [80])

    def test_utmeldt_add_unpaid_giro(self):
        # Adding new (unpaid) giro to utmeldt member doesn't
        # actually get saved
        m = lagMedlem(29, har_betalt=True, utmeldt=True)
        g = Giro(medlem=m, belop=10)
        g.save()
        self.assertEqual(
            [g.belop for g in m.giroar.all()], [80])

    def test_utmeldt_add_paid_giro(self):
        # Adding new (unpaid) giro to utmeldt member doesn't
        # actually get saved
        m = lagMedlem(29, har_betalt=True, utmeldt=True)
        g = Giro(medlem=m, belop=20,
                 innbetalt_belop=20, innbetalt=timezone.now())
        g.save()
        self.assertEqual(
            [g.belop for g in m.giroar.all()], [20, 80])


class MedlemManagerTest(TestCase):
    def setUp(self):
        self.medlemar = lagTestMedlemar()

    def test_alle(self):
        """
        Sjekk at me faktisk fær alle når me spyrr om det
        """
        alle = Medlem.objects.alle().values_list('pk', flat=True)
        self.assertEqual(set([x.pk for x in self.medlemar.values()]), set(alle))

    def test_utmelde(self):
        utmelde = Medlem.objects.utmelde().values_list('fornamn', flat=True)
        self.assertEqual(set([
                "12-utmeld", "12-betalt-utmeld",
                "25-utmeld", "25-betalt-utmeld",
                "26-utmeld", "26-betalt-utmeld",
                "23-innmeldtifjor-utmeldtiaar",
            ]), set(utmelde))

    def test_ikkje_utmelde(self):
        ikkje_utmelde = Medlem.objects.ikkje_utmelde().values_list('fornamn', flat=True)
        self.assertEqual(set([
                "12", "12-betalt",
                "25", "25-betalt",
                "26", "26-betalt",
                "23-betaltifjor", "23-innmeldtifjor", "23-betaltiforfjor",
                "23-betalt-utmeldtnesteaar",
                "23-betaltifjor-utmeldtnesteaar",
                "23-innmeldtiforfjor-utmeldtnesteaar",
            ]), set(ikkje_utmelde))

    def test_betalande(self):
        betalande = Medlem.objects.betalande().values_list('fornamn', flat=True)
        self.assertEqual(set([
                "12-betalt",
                "25-betalt",
                "26-betalt",
                "23-betalt-utmeldtnesteaar",
            ]), set(betalande))

    def test_unge(self):
        unge = Medlem.objects.unge().values_list('fornamn', flat=True)
        self.assertEqual(set([
                "12", "12-betalt",
                "25", "25-betalt",
                "23-betaltifjor", "23-innmeldtifjor", "23-betaltiforfjor",
                "23-betalt-utmeldtnesteaar",
                "23-betaltifjor-utmeldtnesteaar",
                "23-innmeldtiforfjor-utmeldtnesteaar",
            ]), set(unge))

    #def test_potensielt_teljande(self):
    #    potensielt_teljande = Medlem.objects.potensielt_teljande().values_list('fornamn', flat=True)
    #    self.assertEqual(set([
    #            "12",
    #            "25",
    #        ]), set(potensielt_teljande))


    def test_teljande(self):
        teljande = Medlem.objects.teljande().values_list('fornamn', flat=True)
        self.assertEqual(set([
                "12-betalt",
                "25-betalt",
                "23-betalt-utmeldtnesteaar",
            ]), set(teljande))

    def test_interessante(self):
        interessante = Medlem.objects.interessante().values_list('fornamn', flat=True)
        self.assertEqual(set([
                "12-betalt", "12",
                "25-betalt", "25",
                "26-betalt", "26",
                "23-betaltifjor",
                "23-betalt-utmeldtnesteaar",
                "23-betaltifjor-utmeldtnesteaar",
            ]), set(interessante))

    def test_interessante_ifjor(self):
        ifjor = Medlem.objects.interessante(year-1).values_list('fornamn', flat=True)
        self.assertEqual(set([
                "23-betaltifjor", "23-innmeldtifjor", "23-betaltiforfjor",
                "23-betaltifjor-utmeldtnesteaar",
                "23-innmeldtifjor-utmeldtiaar",
            ]), set(ifjor))


class WebTest(TestCase):
    def setUp(self):
        lagTestMedlemar()
        self.client = Client()
        user = User.objects.create_user('test_user', password='pass')
        self.client.force_login(user)

    def test_api_get_members(self):
        response = self.client.get('/api/get_members.json')
        self.assertEquals(response.status_code, 200)
        self.assertEquals(len(response.json()), Medlem.objects.count())

    def test_api_get_members_search_term(self):
        response = self.client.get('/api/get_members.json?term=23-')
        self.assertEquals(response.status_code, 200)

        result = response.json()[0]
        checks = dict(bet=[str(year)], lokallag='(ingen)',
                      namn='23-betalt-utmeldtnesteaar E', alder=23)
        for key, value in checks.items():
            self.assertEquals(result[key], value)
        self.assertEquals(len(response.json()), 7)



# -*- coding: utf-8 -*-
# vim: shiftwidth=4 tabstop=4 expandtab softtabstop=4 ai

# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import

import argparse
import sys

from django.conf import settings
from django.core.management.base import BaseCommand
from gunicorn.app.wsgiapp import WSGIApplication

from medlemssys.config.wsgi import application


class Command(BaseCommand):
    help = "Run the production web server"
    leave_locale_alone = True

    def add_arguments(self, parser):
        parser.add_argument('args', nargs=argparse.REMAINDER)

    def create_parser(self, *args, **kwargs):
        parser = super(Command, self).create_parser(*args, **kwargs)
        parser.parse_args = lambda args: parser.parse_known_args(args)[0]
        return parser

    def handle(self, *args, **options):
        # some argv trickery to reuse max of gunicorn wsgiapp
        # prefer this to a more programmatic way
        sys.argv = sys.argv[1:]
        app = settings.WSGI_APPLICATION.rsplit('.', 1)
        sys.argv.append(':'.join(app))
        WSGIApplication().run()



# -*- coding: utf-8 -*-
# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
from django import template
from django.template.defaultfilters import stringfilter

register = template.Library()

@register.filter
@stringfilter
def split(str,splitter=","):
    return str.split(splitter)

@register.simple_tag
def val_option(medlem, val, not_val):
    selected = ''
    if (all(medlem.val_exists(v) for v in val.split(',')) and
            not any(medlem.val_exists(v) for v in not_val.split(','))):
        selected='selected'
    return u'<option {sel} value="{value}">'.format(sel=selected, value=val)

@register.simple_tag
def val_checked(medlem, val):
    if medlem.val_exists(val):
        return 'checked'
    return ''

@register.simple_tag
def val_exists(medlem, val):
    if medlem.val_exists(val):
        return val
    return ''



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
import datetime
import json
import re
import smtplib
from collections import defaultdict

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.db import IntegrityError
from django.db.models import Q
from django.http import HttpResponse, Http404
from django.shortcuts import render_to_response
from django.template import loader
from django.utils import timezone
from django.views.decorators.clickjacking import xframe_options_exempt
from reversion.models import Version

from medlemssys.medlem import admin
assert admin # Silence pyflakes
from medlemssys.medlem.models import (
    Lokallag, Medlem, LokallagOvervaking, Val)
from medlemssys.statistikk.models import LokallagStat


def update_lokallagstat(time):
    time = time or timezone.now()
    lokallag = Lokallag.objects.all()

    llstat = []
    LokallagStat.objects.filter(veke="{0:%Y-%W}".format(time)).delete()

    for llag in lokallag:
        teljande_list = list(llag.medlem_set.teljande().values_list('pk', flat=True))
        interessante_list = list(llag.medlem_set.interessante().values_list('pk', flat=True))
        try:
            llstat.append(LokallagStat.objects.create(
                    lokallag = llag,
                    veke = "{0:%Y-%W}".format(time),

                    teljande = json.dumps(teljande_list),
                    interessante = json.dumps(interessante_list),

                    n_teljande = llag.medlem_set.teljande().count(),
                    n_interessante = llag.medlem_set.interessante().count(),
                    n_ikkje_utmelde = llag.medlem_set.ikkje_utmelde().count(),
                    n_totalt = llag.medlem_set.count(),
                ))
        except IntegrityError:
            pass # Already have this week

    return llstat


def vervetopp_json(request):
    nye = Medlem.objects.filter(oppretta__year=2012, status='M')

    count = defaultdict(lambda: [])
    for n in nye:
        pers = re.sub("(?i)Verva av ", "", n.innmeldingsdetalj).title()

        count[pers].append(
            dict(namn=str(n),
                lokallag=str(n.lokallag),
                teljande=n.er_teljande(),
                betalt=n.har_betalt()
            ))

    return HttpResponse(unicode(json.dumps(count)).encode('utf8'),
            content_type="application/json; charset=utf-8")

FROM_DATE="2013-08-19"
TO_DATE="2013-09-29"

@xframe_options_exempt
def vervetopp(request):
    if not settings.VERVETOPP:
        raise Http404
    from_ = request.GET.get('from', FROM_DATE)
    to = request.GET.get('to', TO_DATE)
    dt_restrict = dict(innmeldt_dato__gte=from_, innmeldt_dato__lte=to)
    vervarar = Medlem.objects.filter(
        status='M',
        **{'har_verva__%s' % k: v for k, v in dt_restrict.items()}
    ).distinct()
    vervedata = []
    for v in vervarar:
        q = v.har_verva.filter(**dt_restrict)
        teljande = set(m.pk for m in q.teljande())
        pot_teljande = set(m.pk for m in q.potensielt_teljande())
        andre = set(m.pk for m in q).difference(teljande.union(pot_teljande))
        vervedata.append((unicode(v), teljande, pot_teljande, andre))
    vervedata.sort(key=lambda v: [len(k) for k in v[1:]], reverse=True)

    return render_to_response(
        'statistikk/vervetopp-embed.html', dict(objects=vervedata))

@xframe_options_exempt
def vervometer(request):
    if not settings.VERVETOPP:
        raise Http404
    from_ = request.GET.get('from', FROM_DATE)
    to = request.GET.get('to', TO_DATE)
    teljande = Medlem.objects.teljande().filter(
        innmeldt_dato__gte=from_,
        innmeldt_dato__lte=to).distinct().count()
    potensielt_teljande = Medlem.objects.potensielt_teljande().filter(
        innmeldt_dato__gte=from_,
        innmeldt_dato__lte=to).distinct().count()
    def get_int(n, default):
        try:
            return int(request.GET[n])
        except (ValueError, KeyError) as e:
            return default
    start = get_int('start', 1000)
    maal = get_int('maal', 200)

    gjenstaande = maal - (teljande + potensielt_teljande)

    return render_to_response('statistikk/vervometer-embed.html',
                              {
                                 'teljande': teljande,
                                 'potensielt_teljande': potensielt_teljande,
                                 'maal': maal,
                                 'totalt': start+maal,
                                 'gjenstaande': gjenstaande,
                              })
def namn_from_pks(model, val_keys):
    namn = []
    for vk in val_keys:
        namn.append(model.objects.get(pk=vk).namn)
    return namn



def _get_overvakingar():
    for overvak in LokallagOvervaking.objects.filter(
            Q(deaktivert__isnull=True)
            | Q(deaktivert__gt=timezone.now()) ):
        epost_seq = overvak.epostar()
        if not epost_seq:
            # Noone to send to anyway
            continue

        if ((timezone.now() - overvak.sist_oppdatert)
                < datetime.timedelta(days=6, seconds=22*60*60)):
            # Har sendt epost for mindre enn 7 dagar sidan, so ikkje send noko no.
            # TODO: Dette er ein sjukt dårleg måte å gjera dette på, fiks betre
            continue

        sist_oppdatering = overvak.sist_oppdatert
        overvak.sist_oppdatert = timezone.now()

        medlem = overvak.lokallag.medlem_set.alle().filter(oppdatert__gt=sist_oppdatering)
        nye_medlem = list(medlem.filter(oppretta__gt=sist_oppdatering).exclude(status='I'))
        nye_infofolk = list(medlem.filter(oppretta__gt=sist_oppdatering, status='I'))
        # Alle andre, "gamle" medlemar
        medlem = medlem.exclude(oppretta__gt=sist_oppdatering)

        # Finn dei som har flytta til eit ANNA lokallag
        try:
            sist_statistikk = LokallagStat.objects.get(
                                veke="{0:%Y-%W}".format(sist_oppdatering),
                                lokallag=overvak.lokallag)
        except LokallagStat.DoesNotExist:
            sist_statistikk = LokallagStat.objects.filter(
                                oppretta__gt=sist_oppdatering,
                                lokallag=overvak.lokallag
                ).order_by("-oppretta").first()

            #stderr(u"LokallagStat for {0}, {1:%Y-%W} fanst ikkje. Brukar {2}" \
            #            .format(overvak.lokallag,
            #                sist_oppdatering,
            #                sist_statistikk))

        if sist_statistikk:
            medlemar_sist = json.loads(sist_statistikk.interessante)
            medlemar_no = overvak.lokallag.medlem_set.interessante().values_list('pk', flat=True)
            vekkflytta_medlem = Medlem.objects.filter(
                                    pk__in=set(medlemar_sist) - set(medlemar_no),
                                    utmeldt_dato__isnull=True)
        else:
            vekkflytta_medlem = []


        tilflytta_medlem, utmeld_medlem, endra_medlem, ukjend_endring = [], [], [], []
        for m in medlem:
            try:
                old = (Version.objects.get_for_object(m)
                       .filter(revision__date_created__lte=sist_oppdatering)
                       .order_by('-revision__date_created')[0])
            except IndexError:
                continue
            new = Version.objects.get_for_object(m)[0]

            def _changed_field(k):
                if old.field_dict[k] == new.field_dict[k]:
                    return False
                if (old.field_dict[k] in (None, '', 'None') and
                        new.field_dict[k] in (None, '', 'None')):
                    return False
                if (isinstance(old.field_dict[k], basestring) and
                        "".join(old.field_dict[k].split())
                        == "".join(new.field_dict[k].split())):
                    return False
                return True
            changed_keys = filter(_changed_field, new.field_dict.keys())

            def _humanify_pks(field, model):
                if field in changed_keys:
                    old.field_dict[field] = ", ".join(
                        namn_from_pks(model, map(int, old.field_dict[field])))
                    new.field_dict[field] = ", ".join(
                        namn_from_pks(model, map(int, new.field_dict[field])))
            _humanify_pks('val', Val)

            m.changed = [ (k, old.field_dict[k], new.field_dict[k])
                          for k in changed_keys
                          if k not in ["_siste_medlemspengar",
                                       "innmeldingstype",
                                       "oppdatert",
                                       "oppretta",
                                       "nykel",
                                      ]]
            if 'utmeldt_dato' in changed_keys and new.field_dict['utmeldt_dato']:
                utmeld_medlem.append(m)
            elif 'lokallag' in changed_keys:
                tilflytta_medlem.append(m)
            elif 'status' in changed_keys and old.field_dict['status'] == 'I':
                nye_medlem.append(m)
            elif m.changed:
                endra_medlem.append(m)
            else:
                ukjend_endring.append(m)
        if not (len(nye_medlem) + len(nye_infofolk) +
                len(tilflytta_medlem) + len(endra_medlem) +
                len(utmeld_medlem) + len(vekkflytta_medlem)):
            # Ikkje send noko dersom det er ingenting å melda
            continue
        ctx = {
            'nye_medlem': nye_medlem,
            'nye_infofolk': nye_infofolk,
            'endra_medlem': endra_medlem,
            'utmeld_medlem': utmeld_medlem,
            'ukjend_endring': ukjend_endring,
            'tilflytta_medlem': tilflytta_medlem,
            'vekkflytta_medlem': vekkflytta_medlem,
        }
        yield epost_seq, overvak, sist_oppdatering, ctx


def send_overvakingar():
    for epost_seq, overvak, sist_oppdatering, ctx in _get_overvakingar():
        msg = create_overvaking_email(
                epost_seq,
                overvak,
                overvak.lokallag,
                sist_oppdatering,
                **ctx)
        try:
            msg.send()
        except smtplib.SMTPRecipientsRefused:
            # TODO Do logging
            continue
        overvak.save()
    return "Ferdig"


def create_overvaking_email(epost_seq, overvaking, lokallag, sist_oppdatering, **ctx):
        context = ctx.copy()
        context.update({
          'dagar': (timezone.now() - sist_oppdatering).days,
          'lokallag': lokallag,
          'overvaking': overvaking,
        })

        text_content = (loader
                        .get_template('epostar/lokallag_overvaking.txt')
                        .render(context))
        html_content = (loader
                        .get_template('epostar/lokallag_overvaking.html')
                        .render(context))
        emne = (loader
                .get_template('epostar/lokallag_overvaking_emnefelt.txt')
                .render(context))
        msg = EmailMultiAlternatives(" ".join(emne.split())[:-1], text_content, "skriv@nynorsk.no", epost_seq)
        msg.attach_alternative(html_content, "text/html")

        return msg



import os

from logan.runner import run_app

import medlemssys.config.settings.defaults


def generate_settings():
    """
    This command is run when ``default_path`` doesn't exist, or ``init`` is
    run and returns a string representing the default data to put into their
    settings file.
    """
    dir = os.path.dirname(medlemssys.config.settings.defaults.__file__)
    settings = open(os.path.join(dir, 'defaults.py')).read()
    return settings


def main():
    run_app(
        project='medlemssys',
        default_config_path='medlemssys_conf.py',
        default_settings='medlemssys.config.settings.defaults',
        settings_initializer=generate_settings,
        settings_envvar='MEDLEMSSYS_CONF',
    )


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from django.conf import settings
from django.core.exceptions import ValidationError
from rest_framework import serializers

from medlemssys.medlem.models import Medlem


class InnmeldingMedlemSerializer(serializers.ModelSerializer):
    namn = serializers.CharField(write_only=True, required=False)
    token = serializers.CharField(write_only=True)

    class Meta:
        fields = (
            'id', 'fornamn', 'etternamn', 'postnr', 'fodt', 'epost', 'mobnr',
            'stad', 'namn', 'token')
        model = Medlem
        extra_kwargs = {
           'etternamn': {'required': False},
           'fodt': {'required': True},
           'fornamn': {'required': False},
           'id': {'read_only': True},
           'stad': {'read_only': True},
        }

    def validate(self, data):
        if not settings.INNMELDING_TOKEN and not settings.DEBUG or (
                data.pop('token', None) != settings.INNMELDING_TOKEN):
            raise ValidationError(
                'Invalid innmelding token')
        if not ('fornamn' in data or 'etternamn' in data):
            if not data.get('namn'):
                raise ValidationError(
                    'Specify either fornamn+etternamn, or namn')
            namn = data['namn'].split()
            if len(namn) == 1:
                data['fornamn'] = namn.pop()
            else:
                data['etternamn'] = namn.pop()
                data['fornamn'] = ' '.join(namn)
        data.pop('namn', None)
        return data

    def create(self, validated_data):
        medlem = Medlem(**validated_data)
        lags = medlem.proposed_lokallag()
        medlem.lokallag = lags[0] if lags else None
        medlem.status = 'I'  # Infoperson
        medlem.innmeldingstype = 'H'  # Heimesida
        medlem.save()
        return medlem


class MedlemSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Medlem
        fields = ('url', 'fornamn', 'mellomnamn', 'etternamn',
                  'fodt', 'postadr', 'ekstraadr', 'postnr', 'epost', 'mobnr',
                  'borteadr', 'bortepostnr')



# -*- coding: utf-8 -*-
# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
from django.contrib import admin

from models import GiroTemplate

class GiroTemplateAdmin(admin.ModelAdmin):
    list_display = ('subject', 'trigger')

admin.site.register(GiroTemplate, GiroTemplateAdmin)
